@@ -27,6 +27,7 @@
 
 #define MAIN_VIEW_TYPE_INITIAL -1
 #define MAIN_VIEW_DND_ICON_OFFSET 20
+#define MAIN_VIEW_RUBBERBAND_SELECT_TRIGGER_LENGTH 32
 
 struct _GdMainViewPrivate {
   GdMainViewType current_type;
@@ -35,7 +36,16 @@ struct _GdMainViewPrivate {
   GtkWidget *current_view;
   GtkTreeModel *model;
 
+  gboolean track_motion;
+  gboolean rubberband_select;
+  GtkTreePath *rubberband_select_first_path;
+  GtkTreePath *rubberband_select_last_path;
+  int button_down_x;
+  int button_down_y;
+
   gchar *button_press_item_path;
+
+  gchar *last_selected_id;
 };
 
 enum {
@@ -73,6 +83,13 @@ gd_main_view_finalize (GObject *obj)
   GdMainView *self = GD_MAIN_VIEW (obj);
 
   g_free (self->priv->button_press_item_path);
+  g_free (self->priv->last_selected_id);
+
+  if (self->priv->rubberband_select_first_path)
+    gtk_tree_path_free (self->priv->rubberband_select_first_path);
+
+  if (self->priv->rubberband_select_last_path)
+    gtk_tree_path_free (self->priv->rubberband_select_last_path);
 
   G_OBJECT_CLASS (gd_main_view_parent_class)->finalize (obj);
 }
@@ -166,6 +183,7 @@ gd_main_view_class_init (GdMainViewClass *klass)
                       GD_MAIN_VIEW_LIST,
                       GD_MAIN_VIEW_ICON,
                       G_PARAM_READWRITE |
+                      G_PARAM_CONSTRUCT |
                       G_PARAM_STATIC_STRINGS);
 
   properties[PROP_SELECTION_MODE] =
@@ -317,6 +335,31 @@ get_generic (GdMainView *self)
   return NULL;
 }
 
+static void
+do_select_row (GdMainView *self,
+               GtkTreeIter *iter,
+               gboolean value)
+{
+  GtkTreeModel *model;
+  GtkTreeModelFilter *filter;
+  GtkTreeIter my_iter, child_iter;
+
+  model = self->priv->model;
+  my_iter = *iter;
+
+  while (GTK_IS_TREE_MODEL_FILTER (model))
+    {
+      filter = GTK_TREE_MODEL_FILTER (model);
+      gtk_tree_model_filter_convert_iter_to_child_iter (filter, &child_iter, &my_iter);
+      model = gtk_tree_model_filter_get_model (filter);
+      my_iter = child_iter;
+    }
+
+  gtk_list_store_set (GTK_LIST_STORE (model), &my_iter,
+                      GD_MAIN_COLUMN_SELECTED, value,
+                      -1);
+}
+
 static void
 selection_mode_do_select_range (GdMainView *self,
                                 GtkTreeIter *first_element,
@@ -326,14 +369,23 @@ selection_mode_do_select_range (GdMainView *self,
   GtkTreePath *path, *last_path;
   gboolean equal;
 
+  path = gtk_tree_model_get_path (self->priv->model, first_element);
   last_path = gtk_tree_model_get_path (self->priv->model, last_element);
-  iter = *first_element;
+  if (gtk_tree_path_compare (path, last_path) > 0)
+    {
+      gtk_tree_path_free (last_path);
+      last_path = path;
+      iter = *last_element;
+    }
+  else
+    {
+      gtk_tree_path_free (path);
+      iter = *first_element;
+    }
 
   do
     {
-      gtk_list_store_set (GTK_LIST_STORE (self->priv->model), &iter,
-                          GD_MAIN_COLUMN_SELECTED, TRUE,
-                          -1);
+      do_select_row (self, &iter, TRUE);
 
       path = gtk_tree_model_get_path (self->priv->model, &iter);
       equal = (gtk_tree_path_compare (path, last_path) == 0);
@@ -354,50 +406,69 @@ selection_mode_select_range (GdMainView *self,
   GtkTreeIter other;
   gboolean found = FALSE;
   gboolean selected;
+  char *id;
 
-  other = *iter;
-  while (gtk_tree_model_iter_previous (self->priv->model, &other))
+  if (self->priv->last_selected_id != NULL &&
+      gtk_tree_model_get_iter_first (self->priv->model, &other))
     {
-      gtk_tree_model_get (self->priv->model, &other,
-                          GD_MAIN_COLUMN_SELECTED, &selected,
-                          -1);
+      do
+	{
+	  gtk_tree_model_get (self->priv->model, &other,
+			      GD_MAIN_COLUMN_ID, &id,
+			      -1);
+	  if (g_strcmp0 (id, self->priv->last_selected_id) == 0)
+	    {
+	      g_free (id);
+	      found = TRUE;
+	      break;
+	    }
+	  g_free (id);
+	}
+      while (gtk_tree_model_iter_next (self->priv->model, &other));
+    }
 
-      if (selected)
-        {
-          found = TRUE;
-          break;
-        }
+  if (!found)
+    {
+      other = *iter;
+      while (gtk_tree_model_iter_previous (self->priv->model, &other))
+	{
+	  gtk_tree_model_get (self->priv->model, &other,
+			      GD_MAIN_COLUMN_SELECTED, &selected,
+			      -1);
+
+	  if (selected)
+	    {
+	      found = TRUE;
+	      break;
+	    }
+	}
+    }
+
+  if (!found)
+    {
+      other = *iter;
+      while (gtk_tree_model_iter_next (self->priv->model, &other))
+	{
+	  gtk_tree_model_get (self->priv->model, &other,
+			      GD_MAIN_COLUMN_SELECTED, &selected,
+			      -1);
+	  if (selected)
+	    {
+	      found = TRUE;
+	      break;
+	    }
+	}
     }
 
   if (found)
+    selection_mode_do_select_range (self, iter, &other);
+  else
     {
-      selection_mode_do_select_range (self, &other, iter);
-      return;
+      /* no other selected element found, just select the iter */
+      do_select_row (self, &iter, TRUE);
     }
 
-  other = *iter;
-  while (gtk_tree_model_iter_next (self->priv->model, &other))
-    {
-      gtk_tree_model_get (self->priv->model, &other,
-                          GD_MAIN_COLUMN_SELECTED, &selected,
-                          -1);
-      if (selected)
-        {
-          found = TRUE;
-          break;
-        }
-    }
-
-  if (found)
-    {
-      selection_mode_do_select_range (self, iter, &other);
-      return;
-    }
-
-  /* no other selected element found, just select the iter */
-  gtk_list_store_set (GTK_LIST_STORE (self->priv->model), iter,
-                      GD_MAIN_COLUMN_SELECTED, TRUE,
-                      -1);
+  g_signal_emit (self, signals[VIEW_SELECTION_CHANGED], 0);
 }
 
 static gboolean
@@ -407,6 +478,7 @@ toggle_selection_for_path (GdMainView *self,
 {
   gboolean selected;
   GtkTreeIter iter;
+  char *id;
 
   if (self->priv->model == NULL)
     return FALSE;
@@ -420,18 +492,22 @@ toggle_selection_for_path (GdMainView *self,
 
   if (selected)
     {
-      gtk_list_store_set (GTK_LIST_STORE (self->priv->model), &iter,
-                          GD_MAIN_COLUMN_SELECTED, FALSE,
-                          -1);
+      do_select_row (self, &iter, FALSE);
     }
   else if (!selected)
     {
       if (select_range)
         selection_mode_select_range (self, &iter);
       else
-        gtk_list_store_set (GTK_LIST_STORE (self->priv->model), &iter,
-                            GD_MAIN_COLUMN_SELECTED, TRUE,
-                            -1);
+	{
+	  gtk_tree_model_get (self->priv->model, &iter,
+			      GD_MAIN_COLUMN_ID, &id,
+			      -1);
+	  g_free (self->priv->last_selected_id);
+	  self->priv->last_selected_id = id;
+
+          do_select_row (self, &iter, TRUE);
+	}
     }
 
   g_signal_emit (self, signals[VIEW_SELECTION_CHANGED], 0);
@@ -478,6 +554,14 @@ on_button_release_view_mode (GdMainView *self,
   return activate_item_for_path (self, path);
 }
 
+static gboolean
+event_triggers_selection_mode (GdkEventButton *event)
+{
+  return
+    (event->button == 3) ||
+    ((event->button == 1) && (event->state & GDK_CONTROL_MASK));
+}
+
 static gboolean
 on_button_release_event (GtkWidget *view,
                          GdkEventButton *event,
@@ -485,10 +569,12 @@ on_button_release_event (GtkWidget *view,
 {
   GdMainView *self = user_data;
   GdMainViewGeneric *generic = get_generic (self);
-  GtkTreePath *path;
+  GtkTreePath *path, *start_path, *end_path, *tmp_path;
+  GtkTreeIter iter;
   gchar *button_release_item_path;
   gboolean selection_mode;
   gboolean res, same_item = FALSE;
+  gboolean is_selected;
 
   /* eat double/triple click events */
   if (event->type != GDK_BUTTON_RELEASE)
@@ -508,6 +594,54 @@ on_button_release_event (GtkWidget *view,
   g_free (self->priv->button_press_item_path);
   self->priv->button_press_item_path = NULL;
 
+  self->priv->track_motion = FALSE;
+  if (self->priv->rubberband_select)
+    {
+      self->priv->rubberband_select = FALSE;
+      gd_main_view_generic_set_rubberband_range (get_generic (self), NULL, NULL);
+      if (self->priv->rubberband_select_last_path)
+	{
+	  if (!self->priv->selection_mode)
+	    g_signal_emit (self, signals[SELECTION_MODE_REQUEST], 0);
+
+	  start_path = gtk_tree_path_copy (self->priv->rubberband_select_first_path);
+	  end_path = gtk_tree_path_copy (self->priv->rubberband_select_last_path);
+	  if (gtk_tree_path_compare (start_path, end_path) > 0)
+	    {
+	      tmp_path = start_path;
+	      start_path = end_path;
+	      end_path = tmp_path;
+	    }
+
+	  while (gtk_tree_path_compare (start_path, end_path) <= 0)
+	    {
+	      if (gtk_tree_model_get_iter (self->priv->model,
+					   &iter, start_path))
+		{
+		  gtk_tree_model_get (self->priv->model, &iter,
+				      GD_MAIN_COLUMN_SELECTED, &is_selected,
+				      -1);
+                  do_select_row (self, &iter, !is_selected);
+		}
+
+	      gtk_tree_path_next (start_path);
+	    }
+
+          g_signal_emit (self, signals[VIEW_SELECTION_CHANGED], 0);
+
+	  gtk_tree_path_free (start_path);
+	  gtk_tree_path_free (end_path);
+	}
+
+      g_clear_pointer (&self->priv->rubberband_select_first_path,
+		       gtk_tree_path_free);
+      g_clear_pointer (&self->priv->rubberband_select_last_path,
+		       gtk_tree_path_free);
+
+      res = TRUE;
+      goto out;
+    }
+
   if (!same_item)
     {
       res = FALSE;
@@ -518,8 +652,7 @@ on_button_release_event (GtkWidget *view,
 
   if (!selection_mode)
     {
-      if ((event->button == 3) ||
-          ((event->button == 1) && (event->state & GDK_CONTROL_MASK)))
+      if (event_triggers_selection_mode (event))
         {
           g_signal_emit (self, signals[SELECTION_MODE_REQUEST], 0);
           selection_mode = TRUE;
@@ -547,40 +680,106 @@ on_button_press_event (GtkWidget *view,
   GList *selection, *l;
   GtkTreePath *sel_path;
   gboolean found = FALSE;
+  gboolean force_selection;
 
   path = gd_main_view_generic_get_path_at_pos (generic, event->x, event->y);
 
   if (path != NULL)
     self->priv->button_press_item_path = gtk_tree_path_to_string (path);
 
-  if (!self->priv->selection_mode ||
-      path == NULL)
+  force_selection = event_triggers_selection_mode (event);
+  if (!self->priv->selection_mode && !force_selection)
     {
       gtk_tree_path_free (path);
       return FALSE;
     }
 
-  selection = gd_main_view_get_selection (self);
-
-  for (l = selection; l != NULL; l = l->next)
+  if (path && !force_selection)
     {
-      sel_path = l->data;
-      if (gtk_tree_path_compare (path, sel_path) == 0)
-        {
-          found = TRUE;
-          break;
-        }
-    }
+      selection = gd_main_view_get_selection (self);
 
-  if (selection != NULL)
-    g_list_free_full (selection, (GDestroyNotify) gtk_tree_path_free);
+      for (l = selection; l != NULL; l = l->next)
+	{
+	  sel_path = l->data;
+	  if (gtk_tree_path_compare (path, sel_path) == 0)
+	    {
+	      found = TRUE;
+	      break;
+	    }
+	}
+
+      if (selection != NULL)
+	g_list_free_full (selection, (GDestroyNotify) gtk_tree_path_free);
+    }
 
   /* if we did not find the item in the selection, block
    * drag and drop, while in selection mode
    */
-  return !found;
+  if (!found)
+    {
+      self->priv->track_motion = TRUE;
+      self->priv->rubberband_select = FALSE;
+      self->priv->rubberband_select_first_path = NULL;
+      self->priv->rubberband_select_last_path = NULL;
+      self->priv->button_down_x = event->x;
+      self->priv->button_down_y = event->y;
+      return TRUE;
+    }
+  else
+    return FALSE;
 }
 
+static gboolean
+on_motion_event (GtkWidget      *widget,
+		 GdkEventMotion *event,
+		 gpointer user_data)
+{
+  GdMainView *self = user_data;
+  GtkTreePath *path;
+
+  if (self->priv->track_motion)
+    {
+      if (!self->priv->rubberband_select &&
+	  (event->x - self->priv->button_down_x) * (event->x - self->priv->button_down_x) +
+	  (event->y - self->priv->button_down_y) * (event->y - self->priv->button_down_y)  >
+	  MAIN_VIEW_RUBBERBAND_SELECT_TRIGGER_LENGTH * MAIN_VIEW_RUBBERBAND_SELECT_TRIGGER_LENGTH)
+	{
+	  self->priv->rubberband_select = TRUE;
+	  if (self->priv->button_press_item_path)
+	    {
+	      self->priv->rubberband_select_first_path =
+		gtk_tree_path_new_from_string (self->priv->button_press_item_path);
+	    }
+	}
+
+      if (self->priv->rubberband_select)
+	{
+	  path = gd_main_view_generic_get_path_at_pos (get_generic (self), event->x, event->y);
+	  if (path != NULL)
+	    {
+	      if (self->priv->rubberband_select_first_path == NULL)
+		self->priv->rubberband_select_first_path = gtk_tree_path_copy (path);
+
+	      if (self->priv->rubberband_select_last_path == NULL ||
+		  gtk_tree_path_compare (self->priv->rubberband_select_last_path, path) != 0)
+		{
+		  if (self->priv->rubberband_select_last_path)
+		    gtk_tree_path_free (self->priv->rubberband_select_last_path);
+		  self->priv->rubberband_select_last_path = path;
+
+		  gd_main_view_generic_set_rubberband_range (get_generic (self),
+							     self->priv->rubberband_select_first_path,
+							     self->priv->rubberband_select_last_path);
+		}
+	      else
+		gtk_tree_path_free (path);
+	    }
+	}
+    }
+  return FALSE;
+}
+
+
 static void
 on_drag_begin (GdMainViewGeneric *generic,
                GdkDragContext *drag_context,
@@ -637,8 +836,16 @@ static void
 on_view_path_activated (GdMainView *self,
                         GtkTreePath *path)
 {
-  if (self->priv->selection_mode)
-    toggle_selection_for_path (self, path, FALSE);
+  GdkModifierType state;
+
+  gtk_get_current_event_state (&state);
+
+  if (self->priv->selection_mode || (state & GDK_CONTROL_MASK) != 0)
+    {
+      if (!self->priv->selection_mode)
+	g_signal_emit (self, signals[SELECTION_MODE_REQUEST], 0);
+      toggle_selection_for_path (self, path, ((state & GDK_SHIFT_MASK) != 0));
+    }
   else
     activate_item_for_path (self, path);
 }
@@ -730,6 +937,8 @@ gd_main_view_rebuild (GdMainView *self)
                     G_CALLBACK (on_button_press_event), self);
   g_signal_connect (self->priv->current_view, "button-release-event",
                     G_CALLBACK (on_button_release_event), self);
+  g_signal_connect (self->priv->current_view, "motion-notify-event",
+                    G_CALLBACK (on_motion_event), self);
   g_signal_connect_after (self->priv->current_view, "drag-begin",
                           G_CALLBACK (on_drag_begin), self);
   g_signal_connect (self->priv->current_view, "view-selection-changed",
