@@ -94,6 +94,7 @@ const Application = new Lang.Class({
 
     _init: function() {
         this.minersRunning = [];
+        this._activationTimestamp = Gdk.CURRENT_TIME;
 
         Gettext.bindtextdomain('gnome-documents', Path.LOCALE_DIR);
         Gettext.textdomain('gnome-documents');
@@ -284,7 +285,9 @@ const Application = new Lang.Class({
                 this.emitJS('miners-changed', this.minersRunning);
 
                 if (error) {
-                    log('Error updating the cache: ' + error.toString());
+                    if (!error.matches(Gio.IOErrorEnum, Gio.IOErrorEnum.CANCELLED))
+                        log('Error updating the cache: ' + error.toString());
+
                     return;
                 }
 
@@ -481,16 +484,19 @@ const Application = new Lang.Class({
     },
 
     vfunc_activate: function() {
-        if (this._mainWindow)
-            this._mainWindow.window.present();
+        if (this._mainWindow) {
+            this._mainWindow.window.present_with_time(this._activationTimestamp);
+            this._activationTimestamp = Gdk.CURRENT_TIME;
+        }
     },
 
     vfunc_command_line: function(cmdline) {
         let args = cmdline.get_arguments();
-        if (args.indexOf('--no-default-window') == -1)
+        if (args.indexOf('--no-default-window') == -1) {
             this._createWindow();
+            modeController.setWindowMode(WindowMode.WindowMode.OVERVIEW);
+        }
 
-        modeController.setWindowMode(WindowMode.WindowMode.OVERVIEW);
         this.activate();
         return 0;
     },
@@ -521,35 +527,51 @@ const Application = new Lang.Class({
         Mainloop.idle_add(Lang.bind(this, this._clearState));
     },
 
-    _onActivateResult: function(provider, urn, terms) {
+    _onActivateResult: function(provider, urn, terms, timestamp) {
         this._createWindow();
         modeController.setWindowMode(WindowMode.WindowMode.PREVIEW);
-        this.activate();
-
-        searchController.setString(terms.join(' '));
 
         let doc = documentManager.getItemById(urn);
         if (doc) {
-            documentManager.setActiveItem(doc);
+            doActivate.apply(this, [doc]);
         } else {
             let job = new TrackerUtils.SingleItemJob(urn, queryBuilder);
             job.run(Query.QueryFlags.UNFILTERED, Lang.bind(this,
                 function(cursor) {
-                    if (!cursor)
+                    if (cursor)
+                        doc = documentManager.addDocumentFromCursor(cursor);
+
+                    doActivate.apply(this, [doc]);
+                }));
+        }
+
+        function doActivate(doc) {
+            documentManager.setActiveItem(doc);
+
+            this._activationTimestamp = timestamp;
+            this.activate();
+
+            // forward the search terms next time we enter the overview
+            let modeChangeId = modeController.connect('window-mode-changed', Lang.bind(this,
+                function(object, newMode) {
+                    if (newMode != WindowMode.WindowMode.OVERVIEW)
                         return;
 
-                    let doc = documentManager.addDocumentFromCursor(cursor);
-                    documentManager.setActiveItem(doc);
+                    modeController.disconnect(modeChangeId);
+
+                    searchController.setString(terms.join(' '));
+                    this.change_action_state('search', GLib.Variant.new('b', true));
                 }));
         }
     },
 
-    _onLaunchSearch: function(provider, terms) {
+    _onLaunchSearch: function(provider, terms, timestamp) {
         this._createWindow();
         modeController.setWindowMode(WindowMode.WindowMode.OVERVIEW);
         searchController.setString(terms.join(' '));
         this.change_action_state('search', GLib.Variant.new('b', true));
 
+        this._activationTimestamp = timestamp;
         this.activate();
     }
 });
